/*	$Id: misc.c,v 1.6 2005/08/22 21:48:35 phil Exp $ */

/* BLURB gpl

                           Coda File System
                              Release 6

          Copyright (c) 1987-2005 Carnegie Mellon University
                  Additional copyrights listed below

This  code  is  distributed "AS IS" without warranty of any kind under
the terms of the GNU General Public Licence Version 2, as shown in the
file  LICENSE.  The  technical and financial  contributors to Coda are
listed in the file CREDITS.

*/

///////////////////////////////////////////////////////////////////////////////
//
//  misc.c -- miscellaneous routines
//
//  (C) Copyright 1998 OSR Open Systems Resources, Inc.
//  All Rights Reserved
//
//  This software is part of a licensed software product and may
//  only be used or copied in accordance with the terms of that license.
//
//  OSR Open Systems Resources, Inc.
//  105 Route 101A Suite 19
//  Amherst, NH 03031  (603) 595-6500 FAX: (603) 595-6503
//
///////////////////////////////////////////////////////////////////////////////

#define THIS_FILE_TRACE __TRACE_FLUSH__

#include <coda_types.h>
#include <coda.h>
#include <coda_nt.h>
#include "debug.h"

//
// NullDiscardIrpCompletion
//
//  This routine is used to discard an IRP from the completion routine.
//
// Inputs:
//  DeviceObject - not used
//  Irp - the I/O operation being completed
//  Context - not used
//
// Outputs:
//  None.
//
// Returns:
//  STATUS_MORE_PROCESSING_REQUIRED - terminates I/O completion processing.
//
// Notes:
//  The purpose of this routine is to do "cleanup" on I/O operations
//  so we don't queue an APC for the I/O completion...
//
NTSTATUS NullDiscardIrpCompletion(PDEVICE_OBJECT DeviceObject,
                                  PIRP Irp,
                                  PVOID Context)
{
    //
    // We don't use these parameters, so we supress warning messages.
    //
    UNREFERENCED_PARAMETER(DeviceObject);
    UNREFERENCED_PARAMETER(Context);

    //
    // Copy the status information back into the "user" IOSB.
    //
    *Irp->UserIosb = Irp->IoStatus;
    
    //
    // Set the user event - wakes up the mainline code doing this.
    //
    KeSetEvent(Irp->UserEvent, 0, FALSE);
    
    
    //
    // Free the IRP now that we are done with it.
    //
    IoFreeIrp(Irp);
    
    //
    // We return STATUS_MORE_PROCESSING_REQUIRED because this "magic" return value
    // tells the I/O Manager that additional processing will be done by this driver
    // to the IRP - in fact, it might (as it is in this case) already BE done - and
    // the IRP cannot be completed.
    //
    return STATUS_MORE_PROCESSING_REQUIRED;
}


// 
//  Get a number we can use as the UID for venus.
//

int GetNtUID(void)
{
    UCHAR buffer[256];
    PISID sid = (PISID) &buffer[sizeof(TOKEN_USER)];
    ULONG index;
    NTSTATUS code;
    HANDLE handle;
    ULONG tokenInfoLength;
    int uid = 1;  // default value.

    // Check the current thread first.

    code = ZwOpenThreadToken(NtCurrentThread(), TOKEN_READ, TRUE, &handle);
    
    if (code == STATUS_NO_TOKEN) {

        // Since we don't have a thread level token we'll use the process
        // level token.  This is the common case (in fact) since the only
        // time a thread has a token is when it is impersonating.

        code = ZwOpenProcessToken(NtCurrentProcess(), TOKEN_READ, &handle);
    } 

    // Should never have this happen ... but return 1 if there was an error.

    if (code != STATUS_SUCCESS)
	return 1;

    //  Retrieve the user information from the token.  Note that this can be
    //  used to query twice (once to get the size of the needed buffer.)  
    //  For this use we've allocated a buffer that should always be 
    //  large enough.

    code = ZwQueryInformationToken(handle, TokenUser, buffer,
				   sizeof(buffer), &tokenInfoLength);

    // Should never have this happen ... but return 1 if there was an error.

    if (code != STATUS_SUCCESS)
	return 1;


    // Debug printings

    CDEBUG ((D_UID, "GetNtUid: *** BEGIN SID Dump ***\n"));
    
    CDEBUG ((D_UID, "    Caller's SID (Revision %u, SubAuthorityCount %u):\n",
		    sid->Revision,
		    sid->SubAuthorityCount));
    
    CDEBUG ((D_UID, "    IdentifierAuthority = %u-%u-%u-%u-%u-%u\n",
		    sid->IdentifierAuthority.Value[0],
		    sid->IdentifierAuthority.Value[1],
		    sid->IdentifierAuthority.Value[2],
		    sid->IdentifierAuthority.Value[3],
		    sid->IdentifierAuthority.Value[4],
		    sid->IdentifierAuthority.Value[5]));

    if (sid->SubAuthorityCount) {
        CDEBUG ((D_UID, "    SubAuthority ="));

        for (index = 0; index < sid->SubAuthorityCount;index++) {
        
            if (index) {
                CDEBUG ((D_UID, "-"));
            }
        
            CDEBUG ((D_UID, "%u", sid->SubAuthority[index]));

        }
    
        CDEBUG ((D_UID, "\n"));
    }

    if (sid->IdentifierAuthority.Value[0] == 0 &&
	sid->IdentifierAuthority.Value[1] == 0 &&
	sid->IdentifierAuthority.Value[2] == 0 &&
	sid->IdentifierAuthority.Value[3] == 0 &&
	sid->IdentifierAuthority.Value[4] == 0 &&
	sid->IdentifierAuthority.Value[5] == 5 &&
	sid->SubAuthority[0] == 21 &&
	sid->SubAuthorityCount == 5) {
	
	uid = (int) sid->SubAuthority[4];
    }

    CDEBUG ((D_UID, "GetNtUID: returning %d\n", uid));
    
    return uid;
}

